/*
  Tool            Exit
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   TSE v4.41.37a upwards (23 May 2020), all variants
  Version         v1.1.1   19 May 2025 (3)


  This tool works like the "Exit" option in TSE's "File" menu,
  but shows two extra buttons when a changed file is asked to be saved:
    "Yes to all" and "No to all".


  INSTALLATION

    Copy this file to TSE's "mac" directory and compile it there, for example
    by opening it there in TSE and applying the Macro Compile menu.

    In your .ui file (see the Help About menu) either:
    - Define or replace a key definition, like this:
        <Alt X>  ExecMacro('Exit')
    - And/or in the FileMenu menu replace "Exit()" by "ExecMacro('Exit')".
    Then recompile the .ui file and restart TSE.


  MOTIVATION

    This tool was created to solve the problem, that if you had many changed
    files open, that you did not want to save, and you wanted to leave the
    editor, then it took ages to press "No" on all changed files.


  NOTES

  - Like TSE's Exit behavior:
    If the "Exit" tool is escaped or cancelled, then all files are still open,
    and those files are saved that the user already said "Yes" to.

  - Sometimes the Exit tool ends with the start-up menu, and sometimes with
    closing TSE.
    I am not sure about what makes the difference.
    However, in this regard too the tool works the same as TSE's File Exit
    menu, so I am leaving it uninvestigated for now.


  HISTORY

  v1        19 May 2025
    Initial release.

  v1.1      19 May 2025 (2)
    In order to not confuse muscle memory when switching
    between TSE's old Exit option and this new tool,
    the tool's button order was changed from
      [ Yes ] [ Yes to all ] [ No ] [ No to all ] [Cancel]
    to
      [ Yes ] [ No ] [ Yes to all ] [ No to all ] [Cancel]


  v1.1.1    19 May 2025 (3)
    Fix: Now shows each changed file when asking to save it.
*/


proc Main()
  integer i         = 0
  integer macro_id  = 0
  integer save_rest = 0
  integer cancel    = FALSE

  PushLocation()
  macro_id = CreateTempBuffer()
  PopLocation()
  PushLocation()

  PrevFile(_DONT_LOAD_)

  for i = 1 to NumFiles()
    NextFile(_DONT_LOAD_)
    SetBufferInt(Str(i), GetBufferId(), macro_id)

    if Query(BufferFlags) & _LOADED_
      if FileChanged()
        if save_rest
          if save_rest == 1
            SaveFile()
          endif
        else
          UpdateDisplay(_ALL_WINDOWS_REFRESH_)
          case MsgBoxEx(SplitPath(CurrMacroFilename(), _NAME_),
                        'Save changes to "' +
                          SqueezePath(CurrFilename(), _USE_HOME_PATH_) + '"',
                        '[ &Yes ];[ &No ];[ Yes to &all ];[ No &to all ];[&Cancel]')
            when 1
              SaveFile()
            when 2
              NoOp()
            when 3
              SaveFile()
              save_rest = 1
            when 4
              save_rest = 2
            otherwise
              cancel = TRUE
              break
          endcase
        endif
      endif
    endif
  endfor

  if cancel
    PopLocation()
  else
    KillLocation()
    GotoBufferId(macro_id)

    for i = 1 to NumFiles()
      AbandonFile(GetBufferInt(Str(i), macro_id))
    endfor
  endif

  AbandonFile(macro_id)
  PurgeMacro(CurrMacroFilename())
end Main

